/*
 * Tools.java
 *
 * Created on 27. Juni 2001, 22:24
 */

package log;


import java.util.*;
import java.io.*;

/**
 * This abstract class is a library of useful static methods, e.g. extraction
 * of start parameters snd file/directory copy.
 *
 * @author  mike
 * @version 0.1
 */
public abstract class Tools {
    
    /** The string is used to identify a parameter that points to an
     * additional parameter file. */
    public static final String PARAM_FILE_OPTION = "paramFile";
    
    /** The maximum memory size allowed to allocate during one read in fileCopy. */
    public static final int MAX_BYTES_BUFFER = 65536;
    
    
    /** Converts the given command line arguments into a property list,
     * optionally loading an additional property File given by the
     * statement 'paramFile=filename'. If no additional property file
     * is found, the given defaultFilename is
     * used to load a default property file if not null. It is looked
     * up in the actual path (in case of a relative path) and alternatively
     * in the users home directory. If the given args array is null, only
     * the default parameter file is loaded.
     *
     * Precedence order is:
     * 1. args[]
     * 2. additional property file
     * 3. default property file (loaded only if additional property file is absent)
     */
    public static Properties argsToProps(String args[], String defaultFilename) {
        Properties              argProps = new Properties();
        ByteArrayOutputStream   saveArgs = new ByteArrayOutputStream();
        String                  parFilename;
        boolean                 additionalPropsLoaded = false;
        
        if(args != null) {
            if(args.length > 0) {
                // pack props from the command line
                for(int i=0; i<args.length; i++) {
                    
                    StringTokenizer toker = new StringTokenizer(args[i],"=");
                    int             counter=0;
                    String          key = null;
                    String          value = "";
                    
                    while(toker.hasMoreElements()) {
                        if(counter == 0) {
                            key = toker.nextToken();
                        } else {
                            if(counter>1) value += "=";
                            value += toker.nextToken();
                        }
                        counter++;
                    }
                    if(key != null) argProps.put(key, value);
                }
                // save arg properties
                try{
                    argProps.store(saveArgs, null);
                } catch (IOException ex) { System.err.println(ex); }
                // look out for paramFile option, load properties from file
                parFilename = argProps.getProperty(PARAM_FILE_OPTION);
                if(parFilename != null) {
                    try{
                        argProps.load(new FileInputStream(parFilename));
                        additionalPropsLoaded = true;
                    } catch(IOException ex) { System.err.println(ex); }
                }
            }
        }
        if(!additionalPropsLoaded) {
            if(defaultFilename != null) {            	
                try{                	
                    argProps.load(new FileInputStream(defaultFilename));
                } catch(IOException ex) {
                    try{
                        argProps.load(new FileInputStream(Boolean.getBoolean("user.dir") +
                        File.separator + defaultFilename));
                    } catch(IOException ex2) {
                        System.err.println("warning: no default parameter file '"
                        + defaultFilename + "' found.");
                    }
                }
            }
        }
        // reload arg properties to take precedence
        try{
            argProps.load(new ByteArrayInputStream(saveArgs.toByteArray()));
        } catch (IOException ex) { System.err.println(ex); }
        //argProps.list(System.out);
        return argProps;
    }
    
    /** Converts the given command line arguments into a property list,
     * optionally loading an additional parameter File given by the
     * statement 'paramFile=filename'. */
    public static Properties argsToProps(String args[]) {
        return argsToProps(args, null);
    }
    
    /** Unpacks properties from a string into a new property set. The single
     *  properties must be divided by commata characters.
     *  The given string may be surrounded by "". */
    public static Properties unpackPropString(String propString) {
        
        Properties  answer = new Properties();
        String      actualProp;
        
        if(propString == null) return null;
        propString = propString.trim();
        if(propString.length() < 2) return answer;
        // now strip outer "" of
        if((propString.indexOf("\"") == 0) &&
        (propString.lastIndexOf("\"") == (propString.length() - 1))) {
            propString = propString.substring(1, propString.length() - 1);
        }
        StringTokenizer toker = new StringTokenizer(propString,",");
        while(toker.hasMoreElements()) {
            actualProp = toker.nextToken();
            int equalPos = actualProp.indexOf("=");
            if(equalPos > 0) {
                String key = actualProp.substring(0, equalPos);
                String value = "";
                if(actualProp.length() > equalPos + 1) value = actualProp.substring(equalPos + 1);
                answer.put(key, value);
            }
        }
        return answer;
    }
    
    
    /** packs the given properties into an array of strings */
    public static String[] propsToArgs( Properties p ) {
        
        String[] answer = new String[ p.size() ];
        int num = 0;
        String key;
        
        Enumeration keys = p.keys();
        while( keys.hasMoreElements() ) {
            key = (String)keys.nextElement();
            answer[ num ] = key + "=" + p.getProperty( key );
            num++;
        }
        return answer;
    }

    
    /** Simple file copy from 'sourcePath' to 'destPath', if some error occurs,
     *  it is reported by passing the appropriate Exception to the caller. */
    public static void fileCopy(String sourcePath, String destPath)
    throws FileNotFoundException, IOException {
        
        BufferedInputStream     source = new BufferedInputStream(new FileInputStream(sourcePath));
        BufferedOutputStream    dest = new BufferedOutputStream(new FileOutputStream(destPath));
        int                     availBytes = 0;
        byte[]                  copyBuf = new byte[MAX_BYTES_BUFFER];
        
        while((availBytes = source.available()) > 0) {
            if(availBytes > MAX_BYTES_BUFFER) availBytes = MAX_BYTES_BUFFER;
            source.read(copyBuf, 0, availBytes);
            dest.write(copyBuf, 0, availBytes);
        }
        source.close();
        dest.close();
    }
    
    /** Copies the contents of the given source directory to the destination
     *  directory. Both directories must exist. Errors are reported by passing
     *  the appropriate Exception to the caller. */
    public static void dirCopy(String sourceDirPath, String destDirPath)
    throws FileNotFoundException, IOException {
        
        File    sourceDir = new File(sourceDirPath);
        File    destDir = new File(destDirPath);
        String  sourceFiles[] = sourceDir.list();
        
        for(int run=0; run<sourceFiles.length; run++) {
            fileCopy(sourceDirPath + File.separator + sourceFiles[run],
            destDirPath + File.separator + sourceFiles[run]);
        }
    }
    
    /** Recursively deletes every file or directory stored in this directory. */
    public static void clearDirectory(File dir) {
        
        boolean     answer = true;
        
        if(dir != null) {
            if(dir.isDirectory()) {
                String[] fileList = dir.list();
                for(int run=0; run<fileList.length; run++) {
                    File delFile = new File(dir, fileList[run]);
                    if(delFile.isDirectory()) {
                        clearDirectory(dir);
                    }
                    answer = (answer && delFile.delete());
                }                                            
            }
        }                
    }
    
}
